import base64
import hashlib
from urllib import parse

''' 对外用户开放的方法 '''


def urlEncode(s) -> str:
    return parse.quote(s)


def urlDecode(s) -> str:
    return parse.unquote(s)


def md5(s, salt='') -> str:
    s = str(s)
    strs = (s + salt).encode('utf-8')
    return hashlib.md5(strs).hexdigest()


def base64Encode(s) -> bytes:
    s = str(s)
    return base64.b64encode(s.encode('utf-8'))


def base64Decode(s) -> str:
    s = str(s)
    return base64.b64decode(s).decode('utf-8')


def json_to_str(s) -> str:
    ''' 表单转字符串 '''
    return parse.urlencode(s)



import re
import requests
import json

import logging

logger = logging.getLogger(__name__)


class Success:
    code = 'code'
    msg = 'msg'
    data = 'data'
    Success = {code: 0, msg: 'success', data: []}
    TASK_START = {code: 0, msg: 'task in progress'}


# from fake_useragent import UserAgent


session = requests.Session()
session.keep_alive = False


# ua = UserAgent()


def get_html(url):
    # change_user_agent()
    html = session.get(url)
    if html.status_code == 200:
        res = html.text
    else:
        res = False
    return res


def put_data(url, data):
    try:
        flag = True

        if len(data) == 1:
            pass
            # logger.info(f'{target}没有web资产，不发送数量信息')
        else:
            html = session.post(url, data=json.dumps(data))
            if html.status_code == 200:
                logger.debug('向url:{} 发送数据成功,内容为{}'.format(url, data))
                # continue
            else:
                flag = False
        if flag:
            res = True
        else:
            res = False
    except Exception as e:
        if 'Failed to establish a new connection:' in str(e):
            logger.warning(e)
        else:
            logger.warning(e, exc_info=True)
        res = False
    return res


def put_full_data(url, data):
    res = True
    try:
        data = dict(enumerate(data))
        html = session.post(url, json=data)
        if html.status_code == 200:
            logger.debug('向url:{} 发送数据成功,内容为{}'.format(url, data))
        else:
            res = False
    except Exception as e:
        if 'Failed to establish a new connection:' in str(e):
            logger.warning(e)
        else:
            logger.warning(e, exc_info=True)
        res = False
    return res


def put_divide_data(url, data):
    try:
        flag = True
        for res in data:
            html = session.post(url, json=res)
            if html.status_code == 200:
                continue
            else:
                flag = False
        if flag and data:
            logger.debug('向url:{} 发送数据成功,内容为{}'.format(url, data))
            res = True
        else:
            res = False
    except Exception as e:
        if 'Failed to establish a new connection:' in str(e):
            logger.warning(e)
        else:
            logger.warning(e, exc_info=1)
        res = False
    return res


def post(url):
    try:
        res = session.post(url)
        if res.status_code != 200:
            res = False
    except Exception as e:
        if 'Failed to establish a new connection:' in str(e):
            logger.warning(e)
        else:
            logger.warning(e, exc_info=True)
        res = False
    return res


def post_data(url, data):
    # change_user_agent()
    try:
        res = session.post(url, data=str(data).encode('utf-8'))
        if res.status_code != 200:
            res = False
    except Exception as e:
        if 'Failed to establish a new connection:' in str(e):
            logger.warning(e)
        else:
            logger.warning(e, exc_info=1)
        res = False
    return res


def post_task_status(url, task):
    try:
        res = Success.TASK_START
        ips = str(task.ip).replace('，', ',').split(',')
        for ip in ips:
            res['ip'] = ip
            # res = dict2str(res)
            html = session.post(url=url, data=res, timeout=10)
            if html.status_code != 200:
                res = False
    except Exception as e:
        if 'Failed to establish a new connection:' in str(e):
            logger.warning(e)
        else:
            logger.warning(e, exc_info=True)
        res = False
    return res

# def change_user_agent():
#     session.headers['User-Agent'] = ua.random
#     session.headers['X-forwarded-for'] = '49.49.49.49'



import yaml
import platform
import string
import os
import sys
import hashlib
import binascii
import re
import random
from urllib.parse import urlparse, urlunparse, parse_qs, urlencode, urljoin
from .iptool.iptool import is_ip


def check_keys(require_keys, request_keys):
    flag = True
    lost_key = None
    for key in require_keys:
        if not (key in request_keys.keys() and request_keys[key]):
            flag = False
            lost_key = key
    return flag, lost_key


def format_url(url):
    res = urlparse(url)
    scheme = res.scheme + '://' if res.scheme else 'http://'
    if res.scheme and not res.hostname:
        scheme = 'http://' if str(res.port) != '443' else 'https://'
        host = str(res.scheme)
        path = str(res.path)
        if res.scheme == 'http':
            port = str(res.port) if res.port else "80"
        else:
            port = str(res.port) if res.port else "443"
    elif res.scheme:
        if res.scheme == 'http':
            port = str(res.port) if res.port else "80"
        else:
            port = str(res.port) if res.port else "443"
        if '/' in str(res.hostname):
            host = str(res.hostname).split('/')[0]
            path = '/' + ''.join(str(res.hostname).split('/')[0:]) + str(res.path)
        else:
            host = str(res.hostname)
            path = str(res.path)
    else:
        port = str(res.port) if res.port else '80'
        if '/' in str(res.path):
            t = str(res.path).split('/')
            host = str(res.path).split('/')[0]
            path = '/' + ''.join(str(res.path).split('/')[1:])
        else:
            host = res.path
            path = ''
    if is_ip(host):
        if path:
            return scheme + host + path
        else:
            return scheme + host + ":" + port
    else:
        if host.startswith('www.'):
            r = scheme + host
        elif host.startswith('host.docker.internal'):
            if port != '80' and port != '443':
                r = scheme + host + ':' + port
            else:
                r = scheme + host
        else:
            r = scheme + 'www.' + host
        return r + path


def switch_api_from_url(url, api_path):
    """
    更改api，保留其他参数
    :param url: 原始url
    :param api_path: 新的api地址（不需要一开始的/）
    :return:
    """
    res = urlparse(url)
    # scheme, netloc, path, params, query, fragment
    res_com = (res.scheme, res.netloc, api_path, res.params, res.query, res.fragment)
    res = urlunparse(res_com)
    return res


def parse_params_for_url(url, dic):
    """在原有的url基础上增加新的query参数字典"""
    res = urlparse(url)
    query = urlencode(dic)
    res_com = (res.scheme, res.netloc, res.path, res.params, '{}&{}'.format(res.query, query), res.fragment)
    res = urlunparse(res_com)
    return res


def set_vul_task_id(ip, report_url):
    try:
        task = parse_qs(urlparse(report_url).query)['task_id'][0]
        process = parse_qs(urlparse(report_url).query)['process'][0]
        task_id = task + '_' + process + '_' + str(ip)
        return {'vul_task_id': task_id}
    except:
        return {}


def read_yaml(config_path, config_name=''):
    """
    config_path:配置文件路径
    config_name:需要读取的配置内容,空则为全读
    """
    if config_path:
        with open(config_path, 'r', encoding="utf-8") as f:
            conf = yaml.safe_load(f.read())  # yaml.load(f.read())
        if not config_name:
            return conf
        elif config_name in conf.keys():
            return conf[config_name.upper()]
        else:
            raise KeyError('未找到对应的配置信息')
    else:
        raise ValueError('请输入正确的配置名称或配置文件路径')


def read_file_index(file_path):
    with open(file_path, 'r', encoding="utf-8") as f:
        num = f.readline()
    return int(num)


def save_file_index(file_path, cnt):
    with open(file_path, 'w+', encoding="utf-8") as f:
        f.write(str(cnt))


def hex_to_bytes(s: str):
    return binascii.a2b_hex(s)


def md5(s: str, encoding='utf-8'):
    return hashlib.md5(s.encode(encoding=encoding)).hexdigest()


def getattr_value(arr, key):
    return getattr(arr, key) if key in arr else ''


def read_file(file, encoding='utf-8'):
    """
    :param file: 文件名
    :param encoding: 编码，默认utf-8
    :return:
    """
    with open(file, 'r', encoding=encoding) as f:
        return f.read()


def where_is(program):
    """
    Protected method enabling the object to find the full path of a binary
    from its PATH environment variable.
    :param program: name of a binary for which the full path needs to
    be discovered.
    :return: the full path to the binary.
    :todo: add a default path list in case PATH is empty.
    """
    __is_windows = (platform.system() == 'Windows')
    split_char = ';' if __is_windows else ':'
    program = program + '.exe' if __is_windows else program
    paths = os.environ.get('PATH', '').split(split_char)
    for path in paths:
        if (os.path.exists(os.path.join(path, program)) and not
        os.path.isdir(os.path.join(path, program))):
            if not platform.system() == 'Windows':
                return os.path.join(path, program)
            else:
                return '\"' + os.path.join(path, program) + '\"'
    return None


def get_real_link(bath_path: str):
    """
    传入文件的绝对路径，返回文件的源文件的绝对路径
    :param bath_path:
    :return:
    """
    if os.path.islink(bath_path):
        real_link = os.readlink(bath_path)
        bath_path = os.path.abspath(os.path.dirname(bath_path))
        path = os.path.join(bath_path, real_link)
        return get_real_link(path)
    else:
        return bath_path


def is_vars(keys, obj):
    '''
    判断对象中是否存在多个键值
    keys和obj必须都为集合
    return:
        success: Boolean 是否全部包含
        unhave: Dict    不存在keys的集合
    '''
    success = False
    try:
        result = keys.intersection(obj)
        unhave = keys - result
        dis = len(keys) - len(result)
        # 判断是否keys全部存在
        if dis == 0:
            success = True

        return success, unhave
    except AttributeError as e:
        print(e)


def format_name(name):
    name = re.sub('[\s\-]+', '_', name)

    return name.lower()


def qs_strr(s):
    data = {}
    if len(s) > 0:
        for key in s:
            data[key] = s[key][0]

    return data


# 初始化数据
def init_data(keys, value=None):
    v = {}
    for key in keys:
        v[key] = value

    return v


# 将json转化为字符串，\n分割
def json_to_warp_str(json):
    s = ''
    for key in json:
        s += f'{key}: {json[key]}\n'
    return s


# 数据过滤
def data_filter(data):
    data = re.sub(r'\s', '', data)

    return data


def list_to_dict(l):
    ''' list 转 dict类型 '''
    d = {i for i in l}
    return d


def get_filename(file):
    ''' 获取文件名 '''
    return file[:file.find('.')]


def dataToStdout(data, bold=False):
    """
    Writes text to the stdout (console) stream
    """

    sys.stdout.write(data)

    try:
        sys.stdout.flush()
    except IOError:
        pass

    return


def get_parent_paths(path, domain=True):
    '''
    通过一个链接分离出各种目录
    :param path:
    :param domain:
    :return:
    '''
    netloc = ''
    if domain:
        p = urlparse(path)
        path = p.path
        netloc = "{}://{}".format(p.scheme, p.netloc)
    paths = []
    if not path or path[0] != '/':
        return paths
    # paths.append(path)
    if path[-1] == '/':
        paths.append(netloc + path)
    tph = path
    if path[-1] == '/':
        tph = path[:-1]
    while tph:
        tph = tph[:tph.rfind('/') + 1]
        paths.append(netloc + tph)
        tph = tph[:-1]
    return paths


def get_links(content, domain, limit=True):
    '''
    从网页源码中匹配链接
    :param content: html源码
    :param domain: 当前网址domain
    :param limit: 是否限定于此域名
    :return:
    '''
    p = urlparse(domain)
    netloc = "{}://{}{}".format(p.scheme, p.netloc, p.path)
    match = re.findall(r'''(href|src)=["'](.*?)["']''', content, re.S | re.I)
    urls = []
    for i in match:
        _domain = urljoin(netloc, i[1])
        if limit:
            if p.netloc.split(":")[0] not in _domain:
                continue
        urls.append(_domain)
    return urls


def random_str(length=10, chars=string.ascii_lowercase):
    return ''.join(random.sample(chars, length))



from setuptools import setup, find_packages

setup(
    name='yuan-tool',  # 打包后的包文件名
    version='1.01',  # 版本号
    keywords="pip",  # 关键字
    description='some common components for personal use',  # 说明
    long_description="some common components for personal use",  # 详细说明
    # license="MIT Licence",  # 许可
    url='',  # 一般是GitHub项目路径
    author='narukami_yume',
    author_email='100961220@qq.com',
    # packages=find_packages(),     #这个参数是导入目录下的所有__init__.py包
    include_package_data=True,
    platforms="any",
    install_requires=['thrift'],  # 引用到的第三方库
    # py_modules=['pip-test.DoRequest', 'pip-test.GetParams', 'pip-test.ServiceRequest',
    #             'pip-test.ts.constants', 'pip-test.ac.Agent2C',
    #             'pip-test.ts.ttypes', 'pip-test.ac.constants',
    #             'pip-test.__init__'],  # 你要打包的文件，这里用下面这个参数代替
    packages=['yuantool']  # 这个参数是导入目录下的所有__init__.py包
)
